-- select a specified selection set in the session
function ApplySelectionSet(name)

    # get a filtered list of only valid units back from the function
    if not selectionSets[name] then return end
    selectionSets[name] = ValidateUnitsList(selectionSets[name])
    local selection = EntityCategoryFilterDown(categories.ALLUNITS - (categories.FACTORY - categories.MOBILE) , selectionSets[name])
    if table.getsize(selection) == 0 then 
        selection = EntityCategoryFilterDown(categories.FACTORY - categories.MOBILE, selectionSets[name])
        if table.getsize(selection) == 0 then
            AddSelectionSet(name, nil)
            return
        end
    end
    if table.getn(selection) > 0 then
        SelectUnits(selection)
        local unitIDs = {}
        for _, unit in selection do
            table.insert(unitIDs, unit:GetEntityId())
        end
        SimCallback({Func = 'OnControlGroupApply', Args = unitIDs})
    
        # Time the difference between the 2 selection application to
        # determine if this is a double tap selection
        local curTime = GetSystemTimeSeconds()
        local diffTime = curTime - lastSelectionTime
        if diffTime > 1.0 then
            lastSelectionName = nil
        end
        lastSelectionTime = curTime
    
        # If this is a double tap then we want to soom in onto the central unit of the group
        if name == lastSelectionName then
            if selection then
				-- AZ ADDITION
				local zoom = GetCamera("WorldCamera"):GetTargetZoom()
                UIZoomTo(selection)
                local settings = GetCamera("WorldCamera"):SaveSettings()
                settings.Zoom = zoom
                GetCamera("WorldCamera"):RestoreSettings(settings)
            end
            lastSelectionName = nil
        else
            lastSelectionName = name
        end        
       
        # if we are out of units. just set our set to nil
        if table.getn(selection) == 0 then
            selectionSets[name] = nil
        else        
            for i,v in selectionSetCallbacks do
                v(name, selectionSets[name], true)
            end
        end
    end
end

function AppendSetToSelection(name)
    # get a filtered list of only valid units back from the function
    local setID = tostring(name)
    selectionSets[setID] = ValidateUnitsList(selectionSets[setID])
    local selectionSet = EntityCategoryFilterDown(categories.ALLUNITS - categories.FACTORY, selectionSets[setID])
    local curSelection = GetSelectedUnits()
    if curSelection and selectionSet then
        for i, unit in selectionSet do
            table.insert(curSelection, unit)
        end
        SelectUnits(curSelection)
    
        # Time the difference between the 2 selection application to
        # determine if this is a double tap selection
        local curTime = GetSystemTimeSeconds()
        local diffTime = curTime - lastSelectionTime
        if diffTime > 1.0 then
            lastSelectionName = nil
        end
        lastSelectionTime = curTime
    
        # If this is a double tap then we want to soom in onto the central unit of the group
        if name == lastSelectionName then
			-- AZ ADDITION
			local zoom = GetCamera("WorldCamera"):GetTargetZoom()
            UIZoomTo(curSelection)
            local settings = GetCamera("WorldCamera"):SaveSettings()
            settings.Zoom = zoom
            GetCamera("WorldCamera"):RestoreSettings(settings)
        else
            lastSelectionName = name
        end
    elseif selectionSet then
        ApplySelectionSet(setID)
    end
end

function FactorySelection(name)
    # get a filtered list of only valid units back from the function
    local setID = tostring(name)
    selectionSets[setID] = ValidateUnitsList(selectionSets[setID])
    local selectionSet = EntityCategoryFilterDown(categories.FACTORY, selectionSets[setID])
    
    SelectUnits(selectionSet)
    
    # Time the difference between the 2 selection application to
    # determine if this is a double tap selection
    local curTime = GetSystemTimeSeconds()
    local diffTime = curTime - lastSelectionTime
    if diffTime > 1.0 then
        lastSelectionName = nil
    end
    lastSelectionTime = curTime

    # If this is a double tap then we want to soom in onto the central unit of the group
    if name == lastSelectionName then
		-- AZ ADDITION
		local zoom = GetCamera("WorldCamera"):GetTargetZoom()
        UIZoomTo(selectionSet)
        local settings = GetCamera("WorldCamera"):SaveSettings()
        settings.Zoom = zoom
        GetCamera("WorldCamera"):RestoreSettings(settings)
    else
        lastSelectionName = name
    end
end

function ResetSelectionSets(new_sets)
    selectionSets = new_sets
end